@@ -108,6 +108,11 @@ FOLLY_GFLAGS_DEFINE_uint32(
     1000,
     "Maximum number of trials (iterations) executed for each benchmark.");
 
+FOLLY_GFLAGS_DEFINE_bool(
+    bm_list,
+    false,
+    "Print out list of all benchmark test names without running them.");
+
 namespace folly {
 namespace detail {
 
@@ -451,9 +456,16 @@ class BenchmarkResultsPrinter {
 
       string s = datum.name;
       if (s == "-") {
+        // Simply draw a line across the benchmark results
         separator('-');
         continue;
       }
+      if (s[0] == '"') {
+        // Simply print some text. Strips implied quote characters
+        // from the beginning and end of the name.
+        printf("%s\n", s.substr(1, s.length() - 2).c_str());
+        continue;
+      }
       bool useBaseline = false;
       // '%' indicates a relative benchmark.
       if (s[0] == '%') {
@@ -848,6 +860,16 @@ bool BenchmarkingStateBase::useCounters() const {
       });
 }
 
+std::vector<std::string> BenchmarkingStateBase::getBenchmarkList() {
+  std::vector<std::string> bmNames;
+  auto toRun = selectBenchmarksToRun(benchmarks_);
+  for (auto benchmarkRegistration : toRun.benchmarks) {
+    bmNames.push_back(benchmarkRegistration->name);
+  }
+
+  return bmNames;
+}
+
 // static
 folly::StringPiece BenchmarkingStateBase::getGlobalBaselineNameForTests() {
   return kGlobalBenchmarkBaseline;
@@ -894,6 +916,16 @@ std::vector<BenchmarkResult> runBenchmarksWithResults() {
 } // namespace detail
 
 void runBenchmarks() {
+  auto& state = detail::globalBenchmarkState();
+
+  if (FLAGS_bm_list) {
+    auto bmNames = state.getBenchmarkList();
+    for (auto testName : bmNames) {
+      std::cout << testName << std::endl;
+    }
+    return;
+  }
+
   if (FLAGS_bm_profile) {
     printf(
         "WARNING: Running with constant number of iterations. Results might be jittery.\n");
@@ -907,8 +939,6 @@ void runBenchmarks() {
 
   checkRunMode();
 
-  auto& state = detail::globalBenchmarkState();
-
   BenchmarkResultsPrinter printer;
   bool useCounter = state.useCounters();
 
