@@ -18,6 +18,8 @@
 
 #include <array>
 
+#include <folly/lang/SafeAssert.h>
+
 namespace folly {
 namespace detail {
 
@@ -238,7 +240,7 @@ constexpr const std::array<
 
 // Check if ASCII is really ASCII
 using IsAscii =
-    bool_constant<'A' == 65 && 'Z' == 90 && 'a' == 97 && 'z' == 122>;
+    std::bool_constant<'A' == 65 && 'Z' == 90 && 'a' == 97 && 'z' == 122>;
 
 // The code in this file that uses tolower() really only cares about
 // 7-bit ASCII characters, so we can take a nice shortcut here.
@@ -463,6 +465,8 @@ template Expected<float, ConversionCode> str_to_floating<float>(
 template Expected<double, ConversionCode> str_to_floating<double>(
     StringPiece* src) noexcept;
 
+namespace {
+
 /**
  * This class takes care of additional processing needed for signed values,
  * like leading sign character and overflow checks.
@@ -478,7 +482,7 @@ class SignedValueHandler<T, true> {
     if (!std::isdigit(*b)) {
       if (*b == '-') {
         negative_ = true;
-      } else if (UNLIKELY(*b != '+')) {
+      } else if (FOLLY_UNLIKELY(*b != '+')) {
         return ConversionCode::INVALID_LEADING_CHAR;
       }
       ++b;
@@ -503,12 +507,12 @@ class SignedValueHandler<T, true> {
 
       FOLLY_POP_WARNING
 
-      if (UNLIKELY(rv > 0)) {
+      if (FOLLY_UNLIKELY(rv > 0)) {
         return makeUnexpected(ConversionCode::NEGATIVE_OVERFLOW);
       }
     } else {
       rv = T(value);
-      if (UNLIKELY(rv < 0)) {
+      if (FOLLY_UNLIKELY(rv < 0)) {
         return makeUnexpected(ConversionCode::POSITIVE_OVERFLOW);
       }
     }
@@ -530,6 +534,8 @@ class SignedValueHandler<T, false> {
   Expected<T, ConversionCode> finalize(T value) { return value; }
 };
 
+} // namespace
+
 /**
  * String represented as a pair of pointers to char to signed/unsigned
  * integrals. Assumes NO whitespace before or after, and also that the
@@ -546,7 +552,7 @@ inline Expected<Tgt, ConversionCode> digits_to(
   SignedValueHandler<Tgt> sgn;
 
   auto err = sgn.init(b);
-  if (UNLIKELY(err != ConversionCode::SUCCESS)) {
+  if (FOLLY_UNLIKELY(err != ConversionCode::SUCCESS)) {
     return makeUnexpected(err);
   }
 
@@ -682,7 +688,7 @@ Expected<Tgt, ConversionCode> str_to_integral(StringPiece* src) noexcept {
   auto b = src->data(), past = src->data() + src->size();
 
   for (;; ++b) {
-    if (UNLIKELY(b >= past)) {
+    if (FOLLY_UNLIKELY(b >= past)) {
       return makeUnexpected(ConversionCode::EMPTY_INPUT_STRING);
     }
     if ((*b < '\t' || *b > '\r') && *b != ' ') {
@@ -693,13 +699,13 @@ Expected<Tgt, ConversionCode> str_to_integral(StringPiece* src) noexcept {
   SignedValueHandler<Tgt> sgn;
   auto err = sgn.init(b);
 
-  if (UNLIKELY(err != ConversionCode::SUCCESS)) {
+  if (FOLLY_UNLIKELY(err != ConversionCode::SUCCESS)) {
     return makeUnexpected(err);
   }
-  if (is_signed_v<Tgt> && UNLIKELY(b >= past)) {
+  if (is_signed_v<Tgt> && FOLLY_UNLIKELY(b >= past)) {
     return makeUnexpected(ConversionCode::NO_DIGITS);
   }
-  if (UNLIKELY(!isdigit(*b))) {
+  if (FOLLY_UNLIKELY(!isdigit(*b))) {
     return makeUnexpected(ConversionCode::NON_DIGIT_CHAR);
   }
 
@@ -707,7 +713,7 @@ Expected<Tgt, ConversionCode> str_to_integral(StringPiece* src) noexcept {
 
   auto tmp = digits_to<UT>(b, m);
 
-  if (UNLIKELY(!tmp.hasValue())) {
+  if (FOLLY_UNLIKELY(!tmp.hasValue())) {
     return makeUnexpected(
         tmp.error() == ConversionCode::POSITIVE_OVERFLOW ? sgn.overflow()
                                                          : tmp.error());
@@ -763,8 +769,9 @@ ConversionError makeConversionError(ConversionCode code, StringPiece input) {
   static_assert(
       std::is_unsigned<std::underlying_type<ConversionCode>::type>::value,
       "ConversionCode should be unsigned");
-  assert((std::size_t)code < kErrorStrings.size());
-  const ErrorString& err = kErrorStrings[(std::size_t)code];
+  auto index = static_cast<std::size_t>(code);
+  FOLLY_SAFE_CHECK(index < kErrorStrings.size(), "code=", uint64_t(index));
+  const ErrorString& err = kErrorStrings[index];
   if (code == ConversionCode::EMPTY_INPUT_STRING && input.empty()) {
     return {err.string, code};
   }
