@@ -63,11 +63,11 @@
 #include <folly/lang/Bits.h>
 #include <folly/stop_watch.h>
 
-using folly::io::compression::detail::dataStartsWithLE;
-using folly::io::compression::detail::prefixToStringLE;
+using folly::compression::detail::dataStartsWithLE;
+using folly::compression::detail::prefixToStringLE;
 
 namespace folly {
-namespace io {
+namespace compression {
 
 Codec::Codec(CodecType type, Optional<int> /* level */, StringPiece /* name */)
     : type_(type) {}
@@ -2015,5 +2015,5 @@ std::unique_ptr<Codec> getAutoUncompressionCodec(
   return AutomaticCodec::create(
       std::move(customCodecs), std::move(terminalCodec));
 }
-} // namespace io
+} // namespace compression
 } // namespace folly
