@@ -29,7 +29,6 @@
 #include <folly/Likely.h>
 #include <folly/Portability.h>
 #include <folly/lang/Align.h>
-#include <folly/lang/Exception.h>
 #include <folly/synchronization/AtomicRef.h>
 
 namespace folly {
@@ -326,7 +325,7 @@ struct AccessSpreader : private detail::AccessSpreaderBase {
   class CpuCache {
    public:
     unsigned cpu(GlobalState const& s) {
-      if (UNLIKELY(cachedCpuUses_-- == 0)) {
+      if (FOLLY_UNLIKELY(cachedCpuUses_-- == 0)) {
         unsigned cpu;
         s.getcpu.load(std::memory_order_relaxed)(&cpu, nullptr, nullptr);
         cachedCpu_ = cpu % kMaxCpus;
