@@ -396,7 +396,7 @@ TEST(UnboundedQueue, enqDeq) {
 }
 
 template <typename RepFunc>
-uint64_t runBench(const std::string& name, int ops, const RepFunc& repFn) {
+uint64_t runBench(const std::string& name, uint64_t ops, const RepFunc& repFn) {
   uint64_t reps = FLAGS_reps;
   uint64_t min = UINTMAX_MAX;
   uint64_t max = 0;
