@@ -403,23 +403,21 @@ class F14BasicSet {
    * @methodset Modifiers
    */
   FOLLY_ALWAYS_INLINE iterator erase(const_iterator pos) {
-    return eraseInto(pos, [](value_type&&) {});
+    return eraseInto(pos, variadic_noop);
   }
 
   /// Remove a range of elements.
   iterator erase(const_iterator first, const_iterator last) {
-    return eraseInto(first, last, [](value_type&&) {});
+    return eraseInto(first, last, variadic_noop);
   }
 
   /// Remove a specific key.
-  size_type erase(key_type const& key) {
-    return eraseInto(key, [](value_type&&) {});
-  }
+  size_type erase(key_type const& key) { return eraseInto(key, variadic_noop); }
 
   /// Remove a key, using a heterogeneous representation.
   template <typename K>
   EnableHeterogeneousErase<K, size_type> erase(K const& key) {
-    return eraseInto(key, [](value_type&&) {});
+    return eraseInto(key, variadic_noop);
   }
 
   /**
@@ -505,11 +503,22 @@ class F14BasicSet {
   F14HashToken prehash(key_type const& key) const {
     return table_.prehash(key);
   }
+  /// @copydoc prehash
+  F14HashToken prehash(key_type const& key, std::size_t hash) const {
+    return table_.prehash(key, hash);
+  }
 
+  /// @copydoc prehash
   template <typename K>
   EnableHeterogeneousFind<K, F14HashToken> prehash(K const& key) const {
     return table_.prehash(key);
   }
+  /// @copydoc prehash
+  template <typename K>
+  EnableHeterogeneousFind<K, F14HashToken> prehash(
+      K const& key, std::size_t hash) const {
+    return table_.prehash(key, hash);
+  }
 
   /**
    * @overloadbrief Prefetch cachelines associated with a key.
@@ -1159,20 +1168,20 @@ class F14VectorSetImpl : public F14BasicSet<SetPolicyWithDefaults<
 
  public:
   FOLLY_ALWAYS_INLINE iterator erase(const_iterator pos) {
-    return eraseInto(pos, [](value_type&&) {});
+    return eraseInto(pos, variadic_noop);
   }
 
   iterator erase(const_iterator first, const_iterator last) {
-    return eraseInto(first, last, [](value_type&&) {});
+    return eraseInto(first, last, variadic_noop);
   }
 
   std::size_t erase(key_type const& key) {
-    return eraseInto(key, [](value_type&&) {});
+    return eraseInto(key, variadic_noop);
   }
 
   template <typename K>
   EnableHeterogeneousVectorErase<K, std::size_t> erase(K const& key) {
-    return eraseInto(key, [](value_type&&) {});
+    return eraseInto(key, variadic_noop);
   }
 
   template <typename BeforeDestroy>
