@@ -36,7 +36,7 @@
 
 FOLLY_GFLAGS_DEFINE_bool(
     folly_memory_idler_purge_arenas,
-    true,
+    false,
     "if enabled, folly memory-idler purges jemalloc arenas on thread idle");
 
 namespace folly {
@@ -128,7 +128,9 @@ static void fetchStackLimits() {
     tls_stackSize = 1;
     return;
   }
-  SCOPE_EXIT { pthread_attr_destroy(&attr); };
+  SCOPE_EXIT {
+    pthread_attr_destroy(&attr);
+  };
 
   void* addr;
   size_t rawSize;
