@@ -15,6 +15,7 @@
  */
 
 #include <folly/executors/GlobalThreadPoolList.h>
+#include <folly/system/ThreadId.h>
 
 #include <memory>
 #include <string>
@@ -27,17 +28,21 @@
 
 namespace folly {
 
-namespace {
+namespace debugger_detail {
 
 class ThreadListHook {
  public:
-  ThreadListHook(ThreadPoolListHook* poolId, std::thread::id threadId);
+  ThreadListHook(
+      ThreadPoolListHook* poolId,
+      std::thread::id threadId,
+      uint64_t osThreadId);
   ~ThreadListHook();
 
  private:
   ThreadListHook() = default;
   ThreadPoolListHook* poolId_;
   std::thread::id threadId_;
+  uint64_t osThreadId_;
 };
 
 class GlobalThreadPoolListImpl {
@@ -49,7 +54,9 @@ class GlobalThreadPoolListImpl {
   void unregisterThreadPool(ThreadPoolListHook* threadPoolId);
 
   void registerThreadPoolThread(
-      ThreadPoolListHook* threadPoolId, std::thread::id threadId);
+      ThreadPoolListHook* threadPoolId,
+      std::thread::id threadId,
+      uint64_t osThreadId);
 
   void unregisterThreadPoolThread(
       ThreadPoolListHook* threadPoolId, std::thread::id threadId);
@@ -59,6 +66,8 @@ class GlobalThreadPoolListImpl {
     ThreadPoolListHook* addr;
     std::string name;
     std::vector<std::thread::id> threads;
+    // separate data structure for backwards compatibility
+    std::vector<uint64_t> osThreadIds;
   };
 
   struct Pools {
@@ -66,11 +75,10 @@ class GlobalThreadPoolListImpl {
     // property
     std::vector<PoolInfo> poolsInfo_;
 
-    std::vector<std::thread::id>* FOLLY_NULLABLE
-    getThreadVector(void* threadPoolId) {
+    PoolInfo* FOLLY_NULLABLE getPool(void* threadPoolId) {
       for (auto& elem : vector()) {
         if (elem.addr == threadPoolId) {
-          return &elem.threads;
+          return &elem;
         }
       }
 
@@ -85,7 +93,7 @@ class GlobalThreadPoolListImpl {
 
 class GlobalThreadPoolList {
  public:
-  GlobalThreadPoolList() = default;
+  GlobalThreadPoolList() noexcept { debug = this; }
 
   static GlobalThreadPoolList& instance();
 
@@ -94,7 +102,9 @@ class GlobalThreadPoolList {
   void unregisterThreadPool(ThreadPoolListHook* threadPoolId);
 
   void registerThreadPoolThread(
-      ThreadPoolListHook* threadPoolId, std::thread::id threadId);
+      ThreadPoolListHook* threadPoolId,
+      std::thread::id threadId,
+      uint64_t osThreadId);
 
   void unregisterThreadPoolThread(
       ThreadPoolListHook* threadPoolId, std::thread::id threadId);
@@ -103,11 +113,14 @@ class GlobalThreadPoolList {
   void operator=(GlobalThreadPoolList const&) = delete;
 
  private:
+  // Make instance() available to the debugger
+  static GlobalThreadPoolList* debug;
+
   folly::Synchronized<GlobalThreadPoolListImpl> globalListImpl_;
   folly::ThreadLocalPtr<ThreadListHook> threadHook_;
 };
 
-} // namespace
+GlobalThreadPoolList* GlobalThreadPoolList::debug;
 
 GlobalThreadPoolList& GlobalThreadPoolList::instance() {
   static folly::Indestructible<GlobalThreadPoolList> ret;
@@ -125,11 +138,15 @@ void GlobalThreadPoolList::unregisterThreadPool(
 }
 
 void GlobalThreadPoolList::registerThreadPoolThread(
-    ThreadPoolListHook* threadPoolId, std::thread::id threadId) {
+    ThreadPoolListHook* threadPoolId,
+    std::thread::id threadId,
+    uint64_t osThreadId) {
   DCHECK(!threadHook_);
-  threadHook_.reset(std::make_unique<ThreadListHook>(threadPoolId, threadId));
+  threadHook_.reset(
+      std::make_unique<ThreadListHook>(threadPoolId, threadId, osThreadId));
 
-  globalListImpl_.wlock()->registerThreadPoolThread(threadPoolId, threadId);
+  globalListImpl_.wlock()->registerThreadPoolThread(
+      threadPoolId, threadId, osThreadId);
 }
 
 void GlobalThreadPoolList::unregisterThreadPoolThread(
@@ -159,29 +176,44 @@ void GlobalThreadPoolListImpl::unregisterThreadPool(
 }
 
 void GlobalThreadPoolListImpl::registerThreadPoolThread(
-    ThreadPoolListHook* threadPoolId, std::thread::id threadId) {
-  auto vec = pools_.getThreadVector(threadPoolId);
-  if (vec == nullptr) {
+    ThreadPoolListHook* threadPoolId,
+    std::thread::id threadId,
+    uint64_t osThreadId) {
+  auto* pool = pools_.getPool(threadPoolId);
+  if (pool == nullptr) {
     return;
   }
 
-  vec->push_back(threadId);
+  auto& threads = pool->threads;
+  auto& osids = pool->osThreadIds;
+  threads.push_back(threadId);
+  osids.push_back(osThreadId);
 }
 
 void GlobalThreadPoolListImpl::unregisterThreadPoolThread(
     ThreadPoolListHook* threadPoolId, std::thread::id threadId) {
-  auto vec = pools_.getThreadVector(threadPoolId);
-  if (vec == nullptr) {
+  auto* pool = pools_.getPool(threadPoolId);
+  if (pool == nullptr) {
     return;
   }
 
-  vec->erase(std::remove(vec->begin(), vec->end(), threadId), vec->end());
+  auto& threads = pool->threads;
+  auto& osids = pool->osThreadIds;
+  DCHECK_EQ(threads.size(), osids.size());
+  for (unsigned i = 0; i < threads.size(); ++i) {
+    if (threads[i] == threadId) {
+      threads.erase(threads.begin() + i);
+      osids.erase(osids.begin() + i);
+      break;
+    }
+  }
 }
 
 ThreadListHook::ThreadListHook(
-    ThreadPoolListHook* poolId, std::thread::id threadId) {
+    ThreadPoolListHook* poolId, std::thread::id threadId, uint64_t osThreadId) {
   poolId_ = poolId;
   threadId_ = threadId;
+  osThreadId_ = osThreadId;
 }
 
 ThreadListHook::~ThreadListHook() {
@@ -189,17 +221,21 @@ ThreadListHook::~ThreadListHook() {
       poolId_, threadId_);
 }
 
+} // namespace debugger_detail
+
 ThreadPoolListHook::ThreadPoolListHook(std::string name) {
-  GlobalThreadPoolList::instance().registerThreadPool(this, name);
+  debugger_detail::GlobalThreadPoolList::instance().registerThreadPool(
+      this, name);
 }
 
 ThreadPoolListHook::~ThreadPoolListHook() {
-  GlobalThreadPoolList::instance().unregisterThreadPool(this);
+  debugger_detail::GlobalThreadPoolList::instance().unregisterThreadPool(this);
 }
 
 void ThreadPoolListHook::registerThread() {
-  GlobalThreadPoolList::instance().registerThreadPoolThread(
-      this, std::this_thread::get_id());
+  uint64_t ostid = folly::getOSThreadID();
+  debugger_detail::GlobalThreadPoolList::instance().registerThreadPoolThread(
+      this, std::this_thread::get_id(), ostid);
 }
 
 } // namespace folly
