@@ -23,6 +23,9 @@
 #include <folly/Conv.h>
 #include <folly/Memory.h>
 #include <folly/Random.h>
+#include <folly/coro/GtestHelpers.h>
+#include <folly/coro/Timeout.h>
+#include <folly/coro/WithCancellation.h>
 #include <folly/executors/CPUThreadPoolExecutor.h>
 #include <folly/experimental/coro/BlockingWait.h>
 #include <folly/fibers/AddTasks.h>
@@ -1900,6 +1903,85 @@ TEST(FiberManager, semaphore) {
   EXPECT_EQ(sem.getAvailableTokens(), kNumTokens);
 }
 
+#if FOLLY_HAS_COROUTINES
+
+CO_TEST(FiberManager, SemaphoreCoTryWaitForCanCancelOrTimeout) {
+  using namespace std::chrono_literals;
+
+  {
+    // Pre-cancel
+    Semaphore sem{0};
+    folly::CancellationSource cancelSource;
+    cancelSource.requestCancellation();
+    EXPECT_THROW(
+        co_await folly::coro::co_withCancellation(
+            cancelSource.getToken(), sem.co_try_wait_for(1h)),
+        folly::OperationCancelled);
+  }
+  {
+    // Cancel
+    Semaphore sem{0};
+    folly::CancellationSource cancelSource;
+    auto t = std::thread([&]() { cancelSource.requestCancellation(); });
+    EXPECT_THROW(
+        co_await folly::coro::co_withCancellation(
+            cancelSource.getToken(), sem.co_try_wait_for(1h)),
+        folly::OperationCancelled);
+    t.join();
+  }
+  {
+    // Timeout
+    Semaphore sem{0};
+    folly::CancellationSource cancelSource;
+    EXPECT_THROW(
+        co_await folly::coro::co_withCancellation(
+            cancelSource.getToken(), sem.co_try_wait_for(1ms)),
+        folly::OperationCancelled);
+  }
+}
+
+CO_TEST(FiberManager, StressTestSemaphoreCoTryWaitFor) {
+  static constexpr size_t kIterations = 10000;
+
+  Semaphore readyToRead{0};
+  Semaphore sem{0};
+
+  auto t = std::thread([&]() {
+    for (size_t i = 0; i < kIterations; ++i) {
+      readyToRead.wait();
+      sem.signal();
+    }
+  });
+
+  auto waitForSignal = [&](size_t i) -> folly::coro::Task<void> {
+    std::chrono::milliseconds timeout{0};
+    auto increaseTimeout = [&]() {
+      if (timeout.count() == 0) {
+        timeout = std::chrono::milliseconds{1};
+      } else if (timeout < std::chrono::milliseconds{10000}) {
+        timeout *= 2;
+      }
+    };
+    readyToRead.signal();
+    while (true) {
+      try {
+        co_await sem.co_try_wait_for(timeout);
+        co_return;
+      } catch (folly::OperationCancelled&) {
+        increaseTimeout();
+      }
+    }
+  };
+
+  for (size_t i = 0; i < kIterations; ++i) {
+    co_await waitForSignal(i);
+  }
+
+  t.join();
+}
+
+#endif
+
 TEST(FiberManager, batchSemaphore) {
   static constexpr size_t kTasks = 10;
   static constexpr size_t kIterations = 10000;
