@@ -96,26 +96,26 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_TRUE((is_result_type_v<vc, vc() const noexcept>));
   EXPECT_TRUE((is_result_type_v<vc, vc() volatile noexcept>));
   EXPECT_TRUE((is_result_type_v<vc, vc() const volatile noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc()& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() const& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() volatile& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() const volatile& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc()&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() const&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() volatile&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc() const volatile&& noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc() & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc() const & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc() volatile & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc() const volatile & noexcept>));
+  EXPECT_TRUE((is_result_type_v < vc, vc() && noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc() const&& noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc() volatile && noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc() const volatile&& noexcept >));
   EXPECT_TRUE((is_result_type_v<vc, vc(...) noexcept>));
   EXPECT_TRUE((is_result_type_v<vc, vc(...) const noexcept>));
   EXPECT_TRUE((is_result_type_v<vc, vc(...) volatile noexcept>));
   EXPECT_TRUE((is_result_type_v<vc, vc(...) const volatile noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...)& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) const& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) volatile& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) const volatile& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...)&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) const&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) volatile&& noexcept>));
-  EXPECT_TRUE((is_result_type_v<vc, vc(...) const volatile&& noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc(...) & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc(...) const & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc(...) volatile & noexcept>));
+  EXPECT_TRUE((is_result_type_v<vc, vc(...) const volatile & noexcept>));
+  EXPECT_TRUE((is_result_type_v < vc, vc(...) && noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc(...) const&& noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc(...) volatile && noexcept >));
+  EXPECT_TRUE((is_result_type_v < vc, vc(...) const volatile&& noexcept >));
 
   //  is_nothrow
 
@@ -147,26 +147,26 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_TRUE((is_nothrow_v<void() const noexcept>));
   EXPECT_TRUE((is_nothrow_v<void() volatile noexcept>));
   EXPECT_TRUE((is_nothrow_v<void() const volatile noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void()& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() const& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() volatile& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() const volatile& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void()&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() const&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() volatile&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void() const volatile&& noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void() & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void() const & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void() volatile & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void() const volatile & noexcept>));
+  EXPECT_TRUE((is_nothrow_v < void() && noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void() const&& noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void() volatile && noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void() const volatile&& noexcept >));
   EXPECT_TRUE((is_nothrow_v<void(...) noexcept>));
   EXPECT_TRUE((is_nothrow_v<void(...) const noexcept>));
   EXPECT_TRUE((is_nothrow_v<void(...) volatile noexcept>));
   EXPECT_TRUE((is_nothrow_v<void(...) const volatile noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...)& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) const& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) volatile& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) const volatile& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...)&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) const&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) volatile&& noexcept>));
-  EXPECT_TRUE((is_nothrow_v<void(...) const volatile&& noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void(...) & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void(...) const & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void(...) volatile & noexcept>));
+  EXPECT_TRUE((is_nothrow_v<void(...) const volatile & noexcept>));
+  EXPECT_TRUE((is_nothrow_v < void(...) && noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void(...) const&& noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void(...) volatile && noexcept >));
+  EXPECT_TRUE((is_nothrow_v < void(...) const volatile&& noexcept >));
 
   //  variadic
 
@@ -198,26 +198,26 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_FALSE((is_variadic_v<void() const noexcept>));
   EXPECT_FALSE((is_variadic_v<void() volatile noexcept>));
   EXPECT_FALSE((is_variadic_v<void() const volatile noexcept>));
-  EXPECT_FALSE((is_variadic_v<void()& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() const& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() volatile& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() const volatile& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void()&& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() const&& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() volatile&& noexcept>));
-  EXPECT_FALSE((is_variadic_v<void() const volatile&& noexcept>));
+  EXPECT_FALSE((is_variadic_v<void() & noexcept>));
+  EXPECT_FALSE((is_variadic_v<void() const & noexcept>));
+  EXPECT_FALSE((is_variadic_v<void() volatile & noexcept>));
+  EXPECT_FALSE((is_variadic_v<void() const volatile & noexcept>));
+  EXPECT_FALSE((is_variadic_v < void() && noexcept >));
+  EXPECT_FALSE((is_variadic_v < void() const&& noexcept >));
+  EXPECT_FALSE((is_variadic_v < void() volatile && noexcept >));
+  EXPECT_FALSE((is_variadic_v < void() const volatile&& noexcept >));
   EXPECT_TRUE((is_variadic_v<void(...) noexcept>));
   EXPECT_TRUE((is_variadic_v<void(...) const noexcept>));
   EXPECT_TRUE((is_variadic_v<void(...) volatile noexcept>));
   EXPECT_TRUE((is_variadic_v<void(...) const volatile noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...)& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) const& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) volatile& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) const volatile& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...)&& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) const&& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) volatile&& noexcept>));
-  EXPECT_TRUE((is_variadic_v<void(...) const volatile&& noexcept>));
+  EXPECT_TRUE((is_variadic_v<void(...) & noexcept>));
+  EXPECT_TRUE((is_variadic_v<void(...) const & noexcept>));
+  EXPECT_TRUE((is_variadic_v<void(...) volatile & noexcept>));
+  EXPECT_TRUE((is_variadic_v<void(...) const volatile & noexcept>));
+  EXPECT_TRUE((is_variadic_v < void(...) && noexcept >));
+  EXPECT_TRUE((is_variadic_v < void(...) const&& noexcept >));
+  EXPECT_TRUE((is_variadic_v < void(...) volatile && noexcept >));
+  EXPECT_TRUE((is_variadic_v < void(...) const volatile&& noexcept >));
 
   //  argument
 
@@ -249,30 +249,36 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const noexcept>));
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) volatile noexcept>));
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const volatile noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*)& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) volatile& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const volatile& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*)&& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const&& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) volatile&& noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) & noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) const & noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*) volatile & noexcept>));
+  EXPECT_TRUE(
+      (is_arg_type_v<vc*, 1, void(int, vc*) const volatile & noexcept>));
+  EXPECT_TRUE((is_arg_type_v < vc*, 1, void(int, vc*) && noexcept >));
+  EXPECT_TRUE((is_arg_type_v < vc*, 1, void(int, vc*) const&& noexcept >));
+  EXPECT_TRUE((is_arg_type_v < vc*, 1, void(int, vc*) volatile && noexcept >));
   EXPECT_TRUE(( //
-      is_arg_type_v<vc*, 1, void(int, vc*) const volatile&& noexcept>));
+      is_arg_type_v < vc*,
+      1,
+      void(int, vc*) const volatile&& noexcept >));
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) noexcept>));
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) const noexcept>));
   EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) volatile noexcept>));
   EXPECT_TRUE(( //
       is_arg_type_v<vc*, 1, void(int, vc*, ...) const volatile noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...)& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) const& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) volatile& noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) & noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) const & noexcept>));
+  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) volatile & noexcept>));
   EXPECT_TRUE(( //
-      is_arg_type_v<vc*, 1, void(int, vc*, ...) const volatile& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...)&& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) const&& noexcept>));
-  EXPECT_TRUE((is_arg_type_v<vc*, 1, void(int, vc*, ...) volatile&& noexcept>));
+      is_arg_type_v<vc*, 1, void(int, vc*, ...) const volatile & noexcept>));
+  EXPECT_TRUE((is_arg_type_v < vc*, 1, void(int, vc*, ...) && noexcept >));
+  EXPECT_TRUE((is_arg_type_v < vc*, 1, void(int, vc*, ...) const&& noexcept >));
+  EXPECT_TRUE(
+      (is_arg_type_v < vc*, 1, void(int, vc*, ...) volatile && noexcept >));
   EXPECT_TRUE(( //
-      is_arg_type_v<vc*, 1, void(int, vc*, ...) const volatile&& noexcept>));
+      is_arg_type_v < vc*,
+      1,
+      void(int, vc*, ...) const volatile&& noexcept >));
 
   //  arguments
 
@@ -304,29 +310,32 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const noexcept>));
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float) volatile noexcept>));
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const volatile noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float)& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) volatile& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const volatile& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float)&& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const&& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) volatile&& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const volatile&& noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) & noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const & noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) volatile & noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float) const volatile & noexcept>));
+  EXPECT_EQ(3, (args_size_v < void(int, vc*, float) && noexcept >));
+  EXPECT_EQ(3, (args_size_v < void(int, vc*, float) const&& noexcept >));
+  EXPECT_EQ(3, (args_size_v < void(int, vc*, float) volatile && noexcept >));
+  EXPECT_EQ(
+      3, (args_size_v < void(int, vc*, float) const volatile&& noexcept >));
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) noexcept>));
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) const noexcept>));
   EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) volatile noexcept>));
   EXPECT_EQ(
       3, (args_size_v<void(int, vc*, float, ...) const volatile noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...)& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) const& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) volatile& noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) & noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) const & noexcept>));
+  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) volatile & noexcept>));
   EXPECT_EQ(
-      3, (args_size_v<void(int, vc*, float, ...) const volatile& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...)&& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) const&& noexcept>));
-  EXPECT_EQ(3, (args_size_v<void(int, vc*, float, ...) volatile&& noexcept>));
+      3, (args_size_v<void(int, vc*, float, ...) const volatile & noexcept>));
+  EXPECT_EQ(3, (args_size_v < void(int, vc*, float, ...) && noexcept >));
+  EXPECT_EQ(3, (args_size_v < void(int, vc*, float, ...) const&& noexcept >));
   EXPECT_EQ(
-      3, (args_size_v<void(int, vc*, float, ...) const volatile&& noexcept>));
+      3, (args_size_v < void(int, vc*, float, ...) volatile && noexcept >));
+  EXPECT_EQ(
+      3,
+      (args_size_v < void(int, vc*, float, ...) const volatile&& noexcept >));
 
   //  value_like
 
@@ -361,37 +370,39 @@ TEST_F(TraitsTest, function_traits) {
   EXPECT_TRUE((is_cvref_t<int volatile, int, void() volatile noexcept>));
   EXPECT_TRUE(( //
       is_cvref_t<int const volatile, int, void() const volatile noexcept>));
-  EXPECT_TRUE((is_cvref_t<int&, int, void()& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int const&, int, void() const& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int volatile&, int, void() volatile& noexcept>));
+  EXPECT_TRUE((is_cvref_t<int&, int, void() & noexcept>));
+  EXPECT_TRUE((is_cvref_t<int const&, int, void() const & noexcept>));
+  EXPECT_TRUE((is_cvref_t<int volatile&, int, void() volatile & noexcept>));
   EXPECT_TRUE(( //
-      is_cvref_t<int const volatile&, int, void() const volatile& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int&&, int, void()&& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int const&&, int, void() const&& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int volatile&&, int, void() volatile&& noexcept>));
-  EXPECT_TRUE((
-      is_cvref_t<int const volatile&&, int, void() const volatile&& noexcept>));
+      is_cvref_t<int const volatile&, int, void() const volatile & noexcept>));
+  EXPECT_TRUE((is_cvref_t < int&&, int, void()&& noexcept >));
+  EXPECT_TRUE((is_cvref_t < int const&&, int, void() const&& noexcept >));
+  EXPECT_TRUE((is_cvref_t < int volatile&&, int, void() volatile&& noexcept >));
+  EXPECT_TRUE(
+      (is_cvref_t < int const volatile&&,
+       int,
+       void() const volatile&& noexcept >));
   EXPECT_TRUE((is_cvref_t<int, int, void(...) noexcept>));
   EXPECT_TRUE((is_cvref_t<int const, int, void(...) const noexcept>));
   EXPECT_TRUE((is_cvref_t<int volatile, int, void(...) volatile noexcept>));
   EXPECT_TRUE(( //
       is_cvref_t<int const volatile, int, void(...) const volatile noexcept>));
-  EXPECT_TRUE((is_cvref_t<int&, int, void(...)& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int const&, int, void(...) const& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int volatile&, int, void(...) volatile& noexcept>));
+  EXPECT_TRUE((is_cvref_t<int&, int, void(...) & noexcept>));
+  EXPECT_TRUE((is_cvref_t<int const&, int, void(...) const & noexcept>));
+  EXPECT_TRUE((is_cvref_t<int volatile&, int, void(...) volatile & noexcept>));
   EXPECT_TRUE(( //
       is_cvref_t<
           int const volatile&,
           int,
-          void(...) const volatile& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int&&, int, void(...)&& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int const&&, int, void(...) const&& noexcept>));
-  EXPECT_TRUE((is_cvref_t<int volatile&&, int, void(...) volatile&& noexcept>));
+          void(...) const volatile & noexcept>));
+  EXPECT_TRUE((is_cvref_t < int&&, int, void(...)&& noexcept >));
+  EXPECT_TRUE((is_cvref_t < int const&&, int, void(...) const&& noexcept >));
+  EXPECT_TRUE(
+      (is_cvref_t < int volatile&&, int, void(...) volatile&& noexcept >));
   EXPECT_TRUE(( //
-      is_cvref_t<
-          int const volatile&&,
-          int,
-          void(...) const volatile&& noexcept>));
+      is_cvref_t < int const volatile&&,
+      int,
+      void(...) const volatile&& noexcept >));
 }
 
 TEST_F(TraitsTest, function_remove_cvref) {
@@ -428,14 +439,14 @@ TEST_F(TraitsTest, function_remove_cvref) {
   EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const noexcept>>));
   EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) volatile noexcept>>));
   EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const volatile noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*)& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) volatile& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const volatile& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*)&& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const&& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) volatile&& noexcept>>));
-  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const volatile&& noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) & noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const & noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) volatile & noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const volatile & noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) && noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const && noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) volatile && noexcept>>));
+  EXPECT_TRUE((same_v<g, rem_cvref_t<char(int*) const volatile && noexcept>>));
 
   using gv = char(int*, ...) noexcept;
   EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) noexcept>>));
@@ -443,16 +454,16 @@ TEST_F(TraitsTest, function_remove_cvref) {
   EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) volatile noexcept>>));
   EXPECT_TRUE(( //
       same_v<gv, rem_cvref_t<char(int*, ...) const volatile noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...)& noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) const& noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) volatile& noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) & noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) const & noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) volatile & noexcept>>));
   EXPECT_TRUE(( //
-      same_v<gv, rem_cvref_t<char(int*, ...) const volatile& noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...)&& noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) const&& noexcept>>));
-  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) volatile&& noexcept>>));
+      same_v<gv, rem_cvref_t<char(int*, ...) const volatile & noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) && noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) const && noexcept>>));
+  EXPECT_TRUE((same_v<gv, rem_cvref_t<char(int*, ...) volatile && noexcept>>));
   EXPECT_TRUE(( //
-      same_v<gv, rem_cvref_t<char(int*, ...) const volatile&& noexcept>>));
+      same_v<gv, rem_cvref_t<char(int*, ...) const volatile && noexcept>>));
 }
 
 TEST_F(TraitsTest, function_like_value) {
@@ -494,19 +505,19 @@ TEST_F(TraitsTest, function_like_value) {
   EXPECT_TRUE((is_like_v<char(int*) volatile noexcept, int volatile, g>));
   EXPECT_TRUE(( //
       is_like_v<char(int*) const volatile noexcept, int const volatile, g>));
-  EXPECT_TRUE((is_like_v<char(int*)& noexcept, int&, g>));
-  EXPECT_TRUE((is_like_v<char(int*) const& noexcept, int const&, g>));
-  EXPECT_TRUE((is_like_v<char(int*) volatile& noexcept, int volatile&, g>));
+  EXPECT_TRUE((is_like_v<char(int*) & noexcept, int&, g>));
+  EXPECT_TRUE((is_like_v<char(int*) const & noexcept, int const&, g>));
+  EXPECT_TRUE((is_like_v<char(int*) volatile & noexcept, int volatile&, g>));
   EXPECT_TRUE(( //
-      is_like_v<char(int*) const volatile& noexcept, int const volatile&, g>));
-  EXPECT_TRUE((is_like_v<char(int*)&& noexcept, int&&, g>));
-  EXPECT_TRUE((is_like_v<char(int*) const&& noexcept, int const&&, g>));
-  EXPECT_TRUE((is_like_v<char(int*) volatile&& noexcept, int volatile&&, g>));
+      is_like_v<char(int*) const volatile & noexcept, int const volatile&, g>));
+  EXPECT_TRUE((is_like_v < char(int*) && noexcept, int&&, g >));
+  EXPECT_TRUE((is_like_v < char(int*) const&& noexcept, int const&&, g >));
+  EXPECT_TRUE(
+      (is_like_v < char(int*) volatile && noexcept, int volatile&&, g >));
   EXPECT_TRUE(( //
-      is_like_v<
-          char(int*) const volatile&& noexcept,
-          int const volatile&&,
-          g>));
+      is_like_v < char(int*) const volatile&& noexcept,
+      int const volatile&&,
+      g >));
 
   using gv = char(int*, ...) noexcept;
   EXPECT_TRUE((is_like_v<char(int*, ...) noexcept, int, gv>));
@@ -517,22 +528,24 @@ TEST_F(TraitsTest, function_like_value) {
           char(int*, ...) const volatile noexcept,
           int const volatile,
           gv>));
-  EXPECT_TRUE((is_like_v<char(int*, ...)& noexcept, int&, gv>));
-  EXPECT_TRUE((is_like_v<char(int*, ...) const& noexcept, int const&, gv>));
+  EXPECT_TRUE((is_like_v<char(int*, ...) & noexcept, int&, gv>));
+  EXPECT_TRUE((is_like_v<char(int*, ...) const & noexcept, int const&, gv>));
   EXPECT_TRUE(( //
-      is_like_v<char(int*, ...) volatile& noexcept, int volatile&, gv>));
+      is_like_v<char(int*, ...) volatile & noexcept, int volatile&, gv>));
   EXPECT_TRUE(( //
       is_like_v<
-          char(int*, ...) const volatile& noexcept,
+          char(int*, ...) const volatile & noexcept,
           int const volatile&,
           gv>));
-  EXPECT_TRUE((is_like_v<char(int*, ...)&& noexcept, int&&, gv>));
-  EXPECT_TRUE((is_like_v<char(int*, ...) const&& noexcept, int const&&, gv>));
+  EXPECT_TRUE((is_like_v < char(int*, ...) && noexcept, int&&, gv >));
+  EXPECT_TRUE(
+      (is_like_v < char(int*, ...) const&& noexcept, int const&&, gv >));
   EXPECT_TRUE(( //
-      is_like_v<char(int*, ...) volatile&& noexcept, int volatile&&, gv>));
+      is_like_v < char(int*, ...) volatile && noexcept,
+      int volatile&&,
+      gv >));
   EXPECT_TRUE(( //
-      is_like_v<
-          char(int*, ...) const volatile&& noexcept,
-          int const volatile&&,
-          gv>));
+      is_like_v < char(int*, ...) const volatile&& noexcept,
+      int const volatile&&,
+      gv >));
 }
