@@ -45,15 +45,16 @@
 
 namespace folly {
 
-class FOLLY_EXPORT FutureException
-    : public static_what_exception<std::logic_error> {
+class FOLLY_EXPORT FutureException : public std::logic_error {
  public:
-  using static_what_exception<std::logic_error>::static_what_exception;
+  using std::logic_error::logic_error;
+  FutureException() : std::logic_error{""} {}
 };
 
 class FOLLY_EXPORT FutureInvalid : public FutureException {
  public:
-  FutureInvalid() : FutureException(static_lifetime{}, "Future invalid") {}
+  FutureInvalid() = default;
+  char const* what() const noexcept override { return "Future invalid"; }
 };
 
 /// At most one continuation may be attached to any given Future.
@@ -63,42 +64,52 @@ class FOLLY_EXPORT FutureInvalid : public FutureException {
 /// thrown instead.
 class FOLLY_EXPORT FutureAlreadyContinued : public FutureException {
  public:
-  FutureAlreadyContinued()
-      : FutureException(static_lifetime{}, "Future already continued") {}
+  FutureAlreadyContinued() = default;
+  char const* what() const noexcept override {
+    return "Future already continued";
+  }
 };
 
 class FOLLY_EXPORT FutureNotReady : public FutureException {
  public:
-  FutureNotReady() : FutureException(static_lifetime{}, "Future not ready") {}
+  FutureNotReady() = default;
+  char const* what() const noexcept override { return "Future not ready"; }
 };
 
 class FOLLY_EXPORT FutureCancellation : public FutureException {
  public:
-  FutureCancellation()
-      : FutureException(static_lifetime{}, "Future was cancelled") {}
+  FutureCancellation() = default;
+  char const* what() const noexcept override { return "Future was cancelled"; }
 };
 
 class FOLLY_EXPORT FutureTimeout : public FutureException {
  public:
-  FutureTimeout() : FutureException(static_lifetime{}, "Timed out") {}
+  FutureTimeout() = default;
+  char const* what() const noexcept override { return "Timed out"; }
 };
 
 class FOLLY_EXPORT FuturePredicateDoesNotObtain : public FutureException {
  public:
-  FuturePredicateDoesNotObtain()
-      : FutureException(static_lifetime{}, "Predicate does not obtain") {}
+  FuturePredicateDoesNotObtain() = default;
+  char const* what() const noexcept override {
+    return "Predicate does not obtain";
+  }
 };
 
 class FOLLY_EXPORT FutureNoTimekeeper : public FutureException {
  public:
-  FutureNoTimekeeper()
-      : FutureException(static_lifetime{}, "No timekeeper available") {}
+  FutureNoTimekeeper() = default;
+  char const* what() const noexcept override {
+    return "No timekeeper available";
+  }
 };
 
 class FOLLY_EXPORT FutureNoExecutor : public FutureException {
  public:
-  FutureNoExecutor()
-      : FutureException(static_lifetime{}, "No executor provided to via") {}
+  FutureNoExecutor() = default;
+  char const* what() const noexcept override {
+    return "No executor provided to via";
+  }
 };
 
 template <class T>
