@@ -22,6 +22,9 @@
 #include <boost/crc.hpp>
 
 #include <folly/CpuId.h>
+#include <folly/detail/TrapOnAvx512.h>
+#include <folly/external/fast-crc32/avx512_crc32c_v8s3x4.h> // @manual
+#include <folly/external/fast-crc32/sse_crc32c_v8s3x3.h> // @manual
 #include <folly/hash/detail/ChecksumDetail.h>
 
 #if FOLLY_SSE_PREREQ(4, 2)
@@ -72,6 +75,12 @@ bool crc32c_hw_supported() {
   return id.sse42();
 }
 
+bool crc32c_hw_supported_avx512() {
+  static folly::CpuId id;
+  static bool supported = id.avx512vl() && !detail::hasTrapOnAvx512();
+  return supported;
+}
+
 bool crc32_hw_supported() {
   static folly::CpuId id;
   return id.sse42();
@@ -90,6 +99,10 @@ bool crc32c_hw_supported() {
   return false;
 }
 
+bool crc32c_hw_supported_avx512() {
+  return false;
+}
+
 bool crc32_hw_supported() {
   return false;
 }
@@ -132,7 +145,18 @@ uint32_t crc32_sw(
 } // namespace detail
 
 uint32_t crc32c(const uint8_t* data, size_t nbytes, uint32_t startingChecksum) {
+#if defined(FOLLY_ENABLE_AVX512_CRC32C_V8S3X4)
+  if (detail::crc32c_hw_supported_avx512() && nbytes > 4096) {
+    return detail::avx512_crc32c_v8s3x4(data, nbytes, startingChecksum);
+  }
+#endif
+
   if (detail::crc32c_hw_supported()) {
+#if defined(FOLLY_ENABLE_SSE42_CRC32C_V8S3X3)
+    if (nbytes > 4096) {
+      return detail::sse_crc32c_v8s3x3(data, nbytes, startingChecksum);
+    }
+#endif
     return detail::crc32c_hw(data, nbytes, startingChecksum);
   } else {
     return detail::crc32c_sw(data, nbytes, startingChecksum);
