@@ -321,7 +321,6 @@ class AsyncSSLSocket : public AsyncSocket {
         new AsyncSSLSocket(ctx, evb, deferSecurityNegotiation));
   }
 
-#if FOLLY_OPENSSL_HAS_SNI
   /**
    * Create a client AsyncSSLSocket with tlsext_servername in
    * the Client Hello message.
@@ -368,7 +367,6 @@ class AsyncSSLSocket : public AsyncSocket {
     return AsyncSSLSocket::UniquePtr(
         new AsyncSSLSocket(ctx, evb, serverName, deferSecurityNegotiation));
   }
-#endif // FOLLY_OPENSSL_HAS_SNI
 
   /**
    * TODO: implement support for SSL renegotiation.
@@ -687,7 +685,6 @@ class AsyncSSLSocket : public AsyncSocket {
     handshakeTimeout_.detachTimeoutManager();
   }
 
-#if OPENSSL_VERSION_NUMBER >= 0x009080bfL
   /**
    * This function will set the SSL context for this socket to the
    * argument. This should only be used on client SSL Sockets that have
@@ -699,7 +696,6 @@ class AsyncSSLSocket : public AsyncSocket {
    * Detaches the SSL context for this socket.
    */
   void detachSSLContext();
-#endif
 
   /**
    * Returns the original folly::SSLContext associated with this socket.
@@ -713,7 +709,6 @@ class AsyncSSLSocket : public AsyncSocket {
     return ctx_;
   }
 
-#if FOLLY_OPENSSL_HAS_SNI
   /**
    * Switch the SSLContext to continue the SSL handshake.
    * It can only be used in server mode.
@@ -736,7 +731,6 @@ class AsyncSSLSocket : public AsyncSocket {
    * ClientHello message.
    */
   void setServerName(std::string serverName) noexcept;
-#endif // FOLLY_OPENSSL_HAS_SNI
 
   void timeoutExpired(std::chrono::milliseconds timeout) noexcept;
 
@@ -1014,10 +1008,8 @@ class AsyncSSLSocket : public AsyncSocket {
   // It doesn't take effect when it is 0.
   size_t minWriteSize_{1500};
 
-#if FOLLY_OPENSSL_HAS_SNI
   std::shared_ptr<const folly::SSLContext> handshakeCtx_;
   std::string tlsextHostname_;
-#endif
 
   // a key that can be used for caching the established session
   std::string sessionKey_;
