@@ -202,7 +202,9 @@ class EventHandlerOobTest : public ::testing::Test {
     clientThread = std::thread([serverPortFuture = serverReady.get_future(),
                                 clientOps]() mutable {
       int clientFd = socket(AF_INET, SOCK_STREAM, 0);
-      SCOPE_EXIT { close(clientFd); };
+      SCOPE_EXIT {
+        close(clientFd);
+      };
       struct hostent* he{nullptr};
       struct sockaddr_in server;
 
@@ -232,7 +234,9 @@ class EventHandlerOobTest : public ::testing::Test {
   void acceptConn() {
     // make the server.
     int listenfd = socket(AF_INET, SOCK_STREAM, 0);
-    SCOPE_EXIT { close(listenfd); };
+    SCOPE_EXIT {
+      close(listenfd);
+    };
     PCHECK(listenfd != -1) << "unable to open socket";
 
     struct sockaddr_in sin;
