@@ -28,7 +28,9 @@
 #include <folly/Portability.h>
 #include <folly/Traits.h>
 #include <folly/Utility.h>
+#include <folly/lang/Assume.h>
 #include <folly/lang/SafeAssert.h>
+#include <folly/lang/Thunk.h>
 #include <folly/lang/TypeInfo.h>
 
 namespace folly {
@@ -308,6 +310,39 @@ catch_exception(Try&& t, Catch&& c, CatchA&&... a) noexcept(
 #endif
 }
 
+namespace detail {
+
+unsigned int* uncaught_exceptions_ptr() noexcept;
+
+} // namespace detail
+
+/// uncaught_exceptions
+///
+/// An accelerated version of std::uncaught_exceptions.
+///
+/// mimic: std::uncaught_exceptions, c++17
+[[FOLLY_ATTR_GNU_PURE]] FOLLY_EXPORT FOLLY_ALWAYS_INLINE int
+uncaught_exceptions() noexcept {
+#if defined(__APPLE__)
+  return std::uncaught_exceptions();
+#elif defined(_CPPLIB_VER)
+  return std::uncaught_exceptions();
+#elif defined(__has_feature) && !FOLLY_HAS_FEATURE(cxx_thread_local)
+  return std::uncaught_exceptions();
+#else
+  thread_local unsigned int* ct;
+  return to_signed(
+      FOLLY_LIKELY(!!ct) ? *ct : *(ct = detail::uncaught_exceptions_ptr()));
+#endif
+}
+
+/// current_exception
+///
+/// An accelerated version of std::current_exception.
+///
+/// mimic: std::current_exception, c++11
+std::exception_ptr current_exception() noexcept;
+
 namespace detail {
 #if FOLLY_APPLE_IOS
 #if __IPHONE_OS_VERSION_MIN_REQUIRED < __IPHONE_12_0
@@ -418,27 +453,246 @@ T* exception_ptr_get_object(std::exception_ptr const& ptr) noexcept {
     return detail::exception_ptr_catching<T&>(
         ptr, +[](T& ex) { return std::addressof(ex); });
   }
-  auto target = type_info_of<T>();
-  auto object = !target ? nullptr : exception_ptr_get_object(ptr, target);
-  return static_cast<T*>(object);
+  if (!kHasRtti) {
+    return nullptr;
+  }
+  return static_cast<T*>(exception_ptr_get_object(ptr, type_info_of<T>()));
 }
 
+/// exception_ptr_try_get_object_exact_fast
+///
+/// Returns the address of the stored exception as if it were upcast to the
+/// given type, if its concrete type is exactly equal to one of the types passed
+/// in the tag.
+///
+/// May hypothetically fail in cases where multipe type-info objects exist for
+/// any of the given types. Positives are true but negatives may be either true
+/// or false.
+template <typename T, typename... S>
+T* exception_ptr_try_get_object_exact_fast(
+    std::exception_ptr const& ptr, tag_t<S...>) noexcept {
+  static_assert((std::is_convertible_v<S*, T*> && ...));
+  if (!kHasRtti || !ptr || !exception_ptr_access()) {
+    return nullptr;
+  }
+  auto const type = exception_ptr_get_type(ptr);
+  if (!type) {
+    return nullptr;
+  }
+  auto const object = exception_ptr_get_object(ptr);
+  auto const fun = [&](auto const phantom, std::type_info const* const target) {
+    assume(!!object);
+    return type == target ? static_cast<decltype(phantom)>(object) : nullptr;
+  };
+  T* out = nullptr;
+  ((out = fun(static_cast<S*>(nullptr), FOLLY_TYPE_INFO_OF(S))) || ...);
+  return out;
+}
+
+/// exception_ptr_get_object_hint
+///
+/// Returns the address of the stored exception as if it were upcast to the
+/// given type, if it could be upcast to that type.
+///
+/// If its concrete type is exactly equal to one of the types passed in the tag,
+/// this may be faster than exception_ptr_get_object without the hint.
+template <typename T, typename... S>
+T* exception_ptr_get_object_hint(
+    std::exception_ptr const& ptr, tag_t<S...> const hint) noexcept {
+  auto const val = exception_ptr_try_get_object_exact_fast<T>(ptr, hint);
+  return FOLLY_LIKELY(!!val) ? val : exception_ptr_get_object<T>(ptr);
+}
+
+namespace detail {
+
+struct make_exception_ptr_with_arg_ {
+  size_t size = 0;
+  std::type_info const* type = nullptr;
+  void (*ctor)(void*, void*) = nullptr;
+  void (*dtor)(void*) = nullptr;
+
+  template <typename F, typename E>
+  static void make(void* p, void* f) {
+    ::new (p) E((*static_cast<F*>(f))());
+  }
+
+  template <typename F, typename E = decltype(FOLLY_DECLVAL(F&)())>
+  FOLLY_ERASE explicit constexpr make_exception_ptr_with_arg_(tag_t<F>) noexcept
+      : size{sizeof(E)},
+        type{FOLLY_TYPE_INFO_OF(E)},
+        ctor{make<F, E>},
+        dtor{thunk::dtor<E>} {}
+};
+
+std::exception_ptr make_exception_ptr_with_(
+    make_exception_ptr_with_arg_ const&, void*) noexcept;
+
+template <typename F>
+struct make_exception_ptr_with_fn_ {
+  F& f_;
+  FOLLY_ERASE std::exception_ptr operator()() const {
+    return std::make_exception_ptr(f_());
+  }
+};
+
+} // namespace detail
+
+/// make_exception_ptr_with_fn
+/// make_exception_ptr_with
+///
+/// Constructs a std::exception_ptr. On some platforms, this form may be more
+/// efficient than std::make_exception_ptr. In particular, even when the latter
+/// is optimized not actually to throw, catch, and call std::current_exception
+/// internally, it remains specified to take its parameter by-value and to copy
+/// its parameter internally. Many in-practice exception types, including those
+/// which ship with standard libraries implementations, have copy constructors
+/// which may atomically modify refcounts; others may allocate and copy string
+/// data. In the best-case scenario, folly::make_exception_ptr_with may avoid
+/// these costs.
+//
+/// There are three overloads, with overload selection unambiguous.
+/// * A single invocable argument. The argument is invoked and its return value
+///   is the managed exception.
+/// * Variadic arguments, the first of which is in_place_type<E>. An exception
+///   of type E is created in-place with the remaining arguments forwarded to
+///   the constructor of E, and it is the managed exception.
+/// * Two arguments, the first of which is in_place. The argument is moved or
+///   copied and the result is the managed exception. This form is the closest
+///   to std::make_exception_ptr.
+///
+/// Example:
+///
+///   std::exception_ptr eptr = make_exception_ptr_with(
+///       [] { return std::runtime_error("message string"); });
+///
+///   std::exception_ptr eptr = make_exception_ptr_with(
+///       std::in_place_type<std::runtime_error>, "message string");
+///
+///   std::exception_ptr eptr = make_exception_ptr_with(
+///       std::in_place, std::runtime_error("message string");
+///
+/// In each example above, the variable eptr holds a managed exception object of
+/// type std::runtime_error with a message string "message string" that would be
+/// returned by member what().
+///
+/// Note that a managed exception object can have any value type whatsoever; it
+/// is not required to have value type of or inheriting std::exception. This is
+/// the same principle as for throw statements and throw_exception above.
+struct make_exception_ptr_with_fn {
+ private:
+  template <typename R>
+  using make_arg_ = conditional_t<
+      std::is_array<std::remove_reference_t<R>>::value,
+      detail::throw_exception_arg_array_,
+      detail::throw_exception_arg_base_>;
+  template <typename R>
+  using make_arg_t = typename make_arg_<R>::template apply<R>;
+
+  template <typename E, typename... A>
+  auto make(A&&... a) const noexcept {
+    return [&] { return E(static_cast<A&&>(a)...); };
+  }
+
+ public:
+  template <typename F, decltype(FOLLY_DECLVAL(F&)())* = nullptr>
+  std::exception_ptr operator()(F f) const noexcept {
+    if ((kIsGlibcxx || kIsLibcpp) && !kIsApple && !kIsWindows //
+        && kHasRtti && exception_ptr_access()) {
+      static const detail::make_exception_ptr_with_arg_ arg{tag<F>};
+      return detail::make_exception_ptr_with_(arg, &f);
+    }
+    if (kHasExceptions) {
+      return catch_exception(
+          detail::make_exception_ptr_with_fn_<F>{f}, current_exception);
+    }
+    return std::exception_ptr();
+  }
+  template <typename E, typename... A>
+  FOLLY_ERASE std::exception_ptr operator()(
+      std::in_place_type_t<E>, A&&... a) const noexcept {
+    return operator()(make<E, make_arg_t<A&&>...>(static_cast<A&&>(a)...));
+  }
+  template <typename E>
+  FOLLY_ERASE std::exception_ptr operator()(
+      std::in_place_t, E&& e) const noexcept {
+    constexpr auto tag = std::in_place_type<remove_cvref_t<E>>;
+    check_(FOLLY_TYPE_INFO_OF(std::decay_t<E>), FOLLY_TYPE_INFO_OF(e));
+    return operator()(tag, static_cast<E&&>(e));
+  }
+
+ private:
+  FOLLY_ALWAYS_INLINE void check_(
+      std::type_info const* s, std::type_info const* d) const noexcept {
+    FOLLY_SAFE_DCHECK(
+        !s || !d || *s == *d,
+        "mismatched static and dynamic types indicates object slicing");
+  }
+};
+inline constexpr make_exception_ptr_with_fn make_exception_ptr_with{};
+
 //  exception_shared_string
 //
 //  An immutable refcounted string, with the same layout as a pointer, suitable
 //  for use in an exception. Exceptions are intended to cheaply nothrow-copy-
 //  constructible and mostly do not need to optimize moves, and this affects how
 //  exception messages are best stored.
+//
+//  May be constructed with a literal string in a very particular form. If so
+//  constructed, (a literal copy of) the literal string will be held with no
+//  refcount required.
 class exception_shared_string {
  private:
-  static void test_params_(char const*, std::size_t);
+  using format_sig_ = void(void*, char*, std::size_t);
 
-  struct state;
-  state* const state_;
+  template <typename F>
+  using test_format_ =
+      decltype(FOLLY_DECLVAL(F)(static_cast<char*>(nullptr), std::size_t(0)));
+
+  struct literal_state_base {
+    unsigned char pad{0};
+  };
+
+  //  a structure with a compile-time string buffer having an odd address
+  template <std::size_t N>
+  struct alignas(2) literal_state : literal_state_base {
+    using lit = literal_string<char, N>;
+    lit what; // address is offset +1 from alignment 2
+
+    literal_state() = delete;
+    explicit constexpr literal_state(lit const str) noexcept : what{str} {}
+  };
+
+  template <auto V>
+  static inline constexpr auto literal_state_instance = literal_state{V};
+
+  static void test_params_(char const*, std::size_t);
+  template <typename F>
+  static void ffun_(void* f, char* b, std::size_t l) {
+    (*static_cast<F*>(f))(b, l);
+  }
+
+  struct state; // alignment is alignof(void*)
+
+  //  state_ can be either state* or char const*
+  //  - low bit 0: state*
+  //  - low bit 1: char const* to &literal_state::what
+  uintptr_t const state_;
+
+  //  private; the wrapping public ctor passes only static-lifetime constants
+  explicit exception_shared_string(literal_state_base const&) noexcept;
+
+  exception_shared_string(std::size_t, format_sig_&, void*);
 
  public:
+#if FOLLY_CPLUSPLUS >= 202002 && !defined(__NVCC__)
+  template <std::size_t N, literal_string<char, N> Str>
+  explicit exception_shared_string(vtag_t<Str>) noexcept
+      : exception_shared_string(literal_state_instance<Str>) {}
+#endif
+
   explicit exception_shared_string(char const*);
   exception_shared_string(char const*, std::size_t);
+
   template <
       typename String,
       typename = decltype(test_params_(
@@ -446,6 +700,12 @@ class exception_shared_string {
           FOLLY_DECLVAL(String const&).size()))>
   explicit exception_shared_string(String const& str)
       : exception_shared_string{str.data(), str.size()} {}
+
+  template <typename F, decltype((void(test_format_<F&>()), 0)) = 0>
+  exception_shared_string(std::size_t size, F func)
+      : exception_shared_string(
+            size, ffun_<F>, &reinterpret_cast<unsigned char&>(func)) {}
+
   exception_shared_string(exception_shared_string const&) noexcept;
   ~exception_shared_string();
   void operator=(exception_shared_string const&) = delete;
@@ -453,32 +713,4 @@ class exception_shared_string {
   char const* what() const noexcept;
 };
 
-/**
- * A wrapper around a given exception type T that allows to store a
- * static-lifetime string as what() return value, avoiding having to copy it
- * into dedicated allocated storage on construction as most standard exceptions
- * do even if the message is static.
- *
- * The constructor from the base class can still be used, in which case what()
- * is delegated to the base class as well.
- */
-template <class T>
-class static_what_exception : public T {
- protected:
-  struct static_lifetime {};
-
- public:
-  using T::T;
-
-  static_what_exception(static_lifetime, const char* msg)
-      : T(std::string{}), msg_(msg) {}
-
-  const char* what() const noexcept override {
-    return msg_ != nullptr ? msg_ : T::what();
-  }
-
- private:
-  const char* msg_ = nullptr;
-};
-
 } // namespace folly
