@@ -242,18 +242,16 @@ struct MakeUnsafeStringSetLargerSize {
   FOLLY_DECLARE_STRING_RESIZE_WITHOUT_INIT_IMPL(TYPE)
 
 #elif defined(_MSC_VER)
-// MSVC
 
 template <typename Tag, typename T, typename A, A Ptr_Eos>
 struct MakeUnsafeStringSetLargerSize {
   friend void unsafeStringSetLargerSizeImpl(
       std::basic_string<T>& s, std::size_t n) {
-    // _Eos method is public for _MSC_VER <= 1916, private after
-    // s._Eos(n);
     (s.*Ptr_Eos)(n);
   }
 };
 
+#if _MSC_VER < 1939
 #define FOLLY_DECLARE_STRING_RESIZE_WITHOUT_INIT(TYPE)          \
   template void std::basic_string<TYPE>::_Eos(std::size_t);     \
   template struct folly::detail::MakeUnsafeStringSetLargerSize< \
@@ -262,7 +260,15 @@ struct MakeUnsafeStringSetLargerSize {
       void (std::basic_string<TYPE>::*)(std::size_t),           \
       &std::basic_string<TYPE>::_Eos>;                          \
   FOLLY_DECLARE_STRING_RESIZE_WITHOUT_INIT_IMPL(TYPE)
-
+#else
+#define FOLLY_DECLARE_STRING_RESIZE_WITHOUT_INIT(TYPE)          \
+  template struct folly::detail::MakeUnsafeStringSetLargerSize< \
+      FollyMemoryDetailTranslationUnitTag,                      \
+      TYPE,                                                     \
+      void (std::basic_string<TYPE>::*)(std::size_t),           \
+      &std::basic_string<TYPE>::_Eos>;                          \
+  FOLLY_DECLARE_STRING_RESIZE_WITHOUT_INIT_IMPL(TYPE)
+#endif // _MSC_VER < 1939
 #else
 #warning \
     "No implementation for resizeWithoutInitialization of std::basic_string"
@@ -376,24 +382,7 @@ void unsafeVectorSetLargerSize(std::vector<T>& v, std::size_t n) {
 
 #define FOLLY_DECLARE_VECTOR_RESIZE_WITHOUT_INIT(TYPE)
 
-#elif defined(_MSC_VER) && _MSC_VER <= 1916
-// MSVC <= VS2017
-
-template <typename Tag, typename T>
-struct MakeUnsafeVectorSetLargerSize : std::vector<T> {
-  friend void unsafeVectorSetLargerSizeImpl(std::vector<T>& v, std::size_t n) {
-    v._Mylast() += (n - v.size());
-  }
-};
-
-#define FOLLY_DECLARE_VECTOR_RESIZE_WITHOUT_INIT(TYPE)          \
-  template struct folly::detail::MakeUnsafeVectorSetLargerSize< \
-      FollyMemoryDetailTranslationUnitTag,                      \
-      TYPE>;                                                    \
-  FOLLY_DECLARE_VECTOR_RESIZE_WITHOUT_INIT_IMPL(TYPE)
-
-#elif defined(_MSC_VER) && _MSC_VER > 1916
-// MSVC >= VS2019
+#elif defined(_MSC_VER)
 
 template <
     typename Tag,
