@@ -14,9 +14,6 @@
  * limitations under the License.
  */
 
-//
-// Author: andrei.alexandrescu@fb.com
-
 #include <folly/FBString.h>
 
 #include <atomic>
@@ -40,6 +37,45 @@
 using namespace std;
 using namespace folly;
 
+template <typename A, typename B>
+using detect_eq = decltype(FOLLY_DECLVAL(A) == FOLLY_DECLVAL(B));
+template <typename A, typename B>
+using detect_ne = decltype(FOLLY_DECLVAL(A) != FOLLY_DECLVAL(B));
+template <typename A, typename B>
+using detect_lt = decltype(FOLLY_DECLVAL(A) < FOLLY_DECLVAL(B));
+template <typename A, typename B>
+using detect_le = decltype(FOLLY_DECLVAL(A) <= FOLLY_DECLVAL(B));
+template <typename A, typename B>
+using detect_gt = decltype(FOLLY_DECLVAL(A) > FOLLY_DECLVAL(B));
+template <typename A, typename B>
+using detect_ge = decltype(FOLLY_DECLVAL(A) >= FOLLY_DECLVAL(B));
+#if FOLLY_CPLUSPLUS >= 202002
+template <typename A, typename B>
+using detect_3w = decltype(FOLLY_DECLVAL(A) <=> FOLLY_DECLVAL(B));
+#endif
+
+static_assert(!std::is_constructible_v<fbstring, nullptr_t>);
+static_assert(!std::is_assignable_v<fbstring, nullptr_t>);
+
+static_assert(!is_detected_v<detect_eq, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_ne, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_lt, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_le, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_gt, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_ge, fbstring, nullptr_t>);
+
+static_assert(!is_detected_v<detect_eq, nullptr_t, fbstring>);
+static_assert(!is_detected_v<detect_ne, nullptr_t, fbstring>);
+static_assert(!is_detected_v<detect_lt, nullptr_t, fbstring>);
+static_assert(!is_detected_v<detect_le, nullptr_t, fbstring>);
+static_assert(!is_detected_v<detect_gt, nullptr_t, fbstring>);
+static_assert(!is_detected_v<detect_ge, nullptr_t, fbstring>);
+
+#if FOLLY_CPLUSPLUS >= 202002
+static_assert(!is_detected_v<detect_3w, fbstring, nullptr_t>);
+static_assert(!is_detected_v<detect_3w, nullptr_t, fbstring>);
+#endif
+
 namespace {
 
 static const int seed = folly::randomNumberSeed();
@@ -58,7 +94,9 @@ template <class String>
 void randomString(String* toFill, unsigned int maxSize = 1000) {
   assert(toFill);
   toFill->resize(random(0, maxSize));
-  FOR_EACH (i, *toFill) { *i = random('a', 'z'); }
+  FOR_EACH (i, *toFill) {
+    *i = random('a', 'z');
+  }
 }
 
 template <class String, class Integral>
@@ -184,7 +222,9 @@ void clause11_21_4_2_j(String& test) {
   auto size = random(0, 2000);
   String s(size, '\0');
   EXPECT_EQ(s.size(), size);
-  FOR_EACH_RANGE (i, 0, s.size()) { s[i] = random('a', 'z'); }
+  FOR_EACH_RANGE (i, 0, s.size()) {
+    s[i] = random('a', 'z');
+  }
   test = s;
 }
 template <class String>
@@ -193,7 +233,9 @@ void clause11_21_4_2_k(String& test) {
   auto size = random(0, 2000);
   String s(size, '\0');
   EXPECT_EQ(s.size(), size);
-  FOR_EACH_RANGE (i, 0, s.size()) { s[i] = random('a', 'z'); }
+  FOR_EACH_RANGE (i, 0, s.size()) {
+    s[i] = random('a', 'z');
+  }
   test = std::move(s);
   if (std::is_same<String, fbstring>::value) {
     EXPECT_LE(s.size(), 128);
@@ -284,7 +326,7 @@ void clause11_21_4_5(String& test) {
   }
 
   EXPECT_THROW(void(test.at(test.size())), std::out_of_range);
-  EXPECT_THROW(void(as_const(test).at(test.size())), std::out_of_range);
+  EXPECT_THROW(void(std::as_const(test).at(test.size())), std::out_of_range);
 }
 
 template <class String>
@@ -298,7 +340,9 @@ void clause11_21_4_6_1(String& test) {
   auto len = test.size();
   test += test1;
   EXPECT_EQ(test.size(), test1.size() + len);
-  FOR_EACH_RANGE (i, 0, test1.size()) { EXPECT_EQ(test[len + i], test1[i]); }
+  FOR_EACH_RANGE (i, 0, test1.size()) {
+    EXPECT_EQ(test[len + i], test1[i]);
+  }
   // aliasing modifiers
   String test2 = test;
   auto dt = test2.data();
@@ -1089,8 +1133,8 @@ TEST(FBString, testAllClauses) {
       rng = RandomT(localSeed);
       f_fbstring(c);
       EXPECT_EQ(r, c) << "Lengths: " << r.size() << " vs. " << c.size()
-                      << "\nReference: '" << r << "'"
-                      << "\nActual:    '" << c.data()[0] << "'";
+                      << "\nReference: '" << r << "'" << "\nActual:    '"
+                      << c.data()[0] << "'";
 #if FOLLY_HAVE_WCHAR_SUPPORT
       rng = RandomT(localSeed);
       f_wfbstring(wc);
@@ -1290,7 +1334,7 @@ TEST(FBString, testMoveOperatorPlusRhs) {
 //      other than libstdc++. Someday if we deem it important to present
 //      identical undefined behavior for other platforms, we can re-visit this.
 TEST(FBString, testConstructionFromLiteralZero) {
-  EXPECT_THROW(fbstring s(nullptr), std::logic_error);
+  EXPECT_THROW(fbstring s(static_cast<const char*>(nullptr)), std::logic_error);
 }
 
 TEST(FBString, testFixedBugsD479397) {
@@ -1726,9 +1770,37 @@ TEST(WFBString, compareToStdWStringLong) {
 }
 #endif
 
-#if FOLLY_HAS_STRING_VIEW
 struct custom_traits : public std::char_traits<char> {};
 
+TEST(FBString, convertFromStringView) {
+  {
+    folly::fbstring test{std::string_view("foo")};
+    std::string control{std::string_view("foo")};
+    EXPECT_EQ(test, "foo");
+    EXPECT_EQ(test, control);
+  }
+  {
+    folly::fbstring test{std::string_view("abcfooabc"), 3, 3};
+    std::string control{std::string_view("abcfooabc"), 3, 3};
+    EXPECT_EQ(test, "foo");
+    EXPECT_EQ(test, control);
+  }
+  {
+    using sv_type = std::basic_string_view<char, custom_traits>;
+    folly::basic_fbstring<char, custom_traits> test{sv_type("foo")};
+    std::basic_string<char, custom_traits> control{sv_type("foo")};
+    EXPECT_EQ(test, "foo");
+    EXPECT_EQ(test, control);
+  }
+  {
+    using sv_type = std::basic_string_view<char, custom_traits>;
+    folly::basic_fbstring<char, custom_traits> test{sv_type("abcfooabc"), 3, 3};
+    std::basic_string<char, custom_traits> control{sv_type("abcfooabc"), 3, 3};
+    EXPECT_EQ(test, "foo");
+    EXPECT_EQ(test, control);
+  }
+}
+
 TEST(FBString, convertToStringView) {
   folly::fbstring s("foo");
   std::string_view sv = s;
@@ -1737,7 +1809,6 @@ TEST(FBString, convertToStringView) {
   std::basic_string_view<char, custom_traits> sv2 = s2;
   EXPECT_EQ(sv2, "bar");
 }
-#endif
 
 TEST(FBString, Format) {
   EXPECT_EQ("  foo", fmt::format("{:>5}", folly::fbstring("foo")));
