@@ -14,8 +14,6 @@
  * limitations under the License.
  */
 
-// @author Nicholas Ormrod <njormrod@fb.com>
-
 /*
 
 This file contains an extensive STL compliance test suite for an STL vector
@@ -164,7 +162,7 @@ THOUGHTS:
 #define VECTOR_ folly::fbvector
 #endif
 
-//#define USING_STD_VECTOR
+// #define USING_STD_VECTOR
 
 #include <climits>
 #include <cstddef>
@@ -394,7 +392,7 @@ int Ticker::TicksLeft = -1;
 
 template <Flags f>
 struct DataTicker : Ticker {
-  DataTicker() noexcept(f& DC_NOEXCEPT) {
+  DataTicker() noexcept(f & DC_NOEXCEPT) {
     if (!(f & DC_NOEXCEPT)) {
       Tick("Data()");
     }
@@ -612,8 +610,8 @@ struct Data : DataTracker<(f & IS_RELOCATABLE) != 0>,
 
 namespace folly {
 template <Flags f, size_t pad>
-struct IsRelocatable<Data<f, pad>> : bool_constant<(f & IS_RELOCATABLE) != 0> {
-};
+struct IsRelocatable<Data<f, pad>>
+    : std::bool_constant<(f & IS_RELOCATABLE) != 0> {};
 } // namespace folly
 
 //-----------------------------------------------------------------------------
@@ -623,17 +621,17 @@ struct IsRelocatable<Data<f, pad>> : bool_constant<(f & IS_RELOCATABLE) != 0> {
 template <typename T>
 struct isPropCopy : true_type {};
 template <Flags f, size_t pad>
-struct isPropCopy<Data<f, pad>> : bool_constant<(f & PROP_COPY) != 0> {};
+struct isPropCopy<Data<f, pad>> : std::bool_constant<(f & PROP_COPY) != 0> {};
 
 template <typename T>
 struct isPropMove : true_type {};
 template <Flags f, size_t pad>
-struct isPropMove<Data<f, pad>> : bool_constant<(f & PROP_MOVE) != 0> {};
+struct isPropMove<Data<f, pad>> : std::bool_constant<(f & PROP_MOVE) != 0> {};
 
 template <typename T>
 struct isPropSwap : true_type {};
 template <Flags f, size_t pad>
-struct isPropSwap<Data<f, pad>> : bool_constant<(f & PROP_SWAP) != 0> {};
+struct isPropSwap<Data<f, pad>> : std::bool_constant<(f & PROP_SWAP) != 0> {};
 
 struct AllocTracker {
   static int Constructed;
@@ -805,19 +803,19 @@ void isSane() {
 
 template <typename T>
 struct is_copy_constructibleAndAssignable
-    : bool_constant<
+    : std::bool_constant<
           std::is_copy_constructible<T>::value &&
           std::is_copy_assignable<T>::value> {};
 
 template <typename T>
 struct is_move_constructibleAndAssignable
-    : bool_constant<
+    : std::bool_constant<
           std::is_move_constructible<T>::value &&
           std::is_move_assignable<T>::value> {};
 
 template <class Vector>
 struct customAllocator
-    : bool_constant<!is_same<
+    : std::bool_constant<!is_same<
           typename Vector::allocator_type,
           std::allocator<typename Vector::value_type>>::value> {};
 
@@ -825,7 +823,7 @@ template <typename T>
 struct special_move_assignable : is_move_constructibleAndAssignable<T> {};
 template <Flags f, size_t pad>
 struct special_move_assignable<Data<f, pad>>
-    : bool_constant<
+    : std::bool_constant<
           is_move_constructibleAndAssignable<Data<f, pad>>::value ||
           f & PROP_MOVE> {};
 
@@ -917,7 +915,7 @@ uint64_t ReadTSC() {
     BOOST_PP_SEQ_FOR_EACH(GEN_CLOSER, _, BOOST_PP_SEQ_REVERSE(argseq))     \
   }                                                                        \
   template <class Vector> void test_ ## name ## 3 () {                     \
-    test_ ## name ## 2 <Vector> (bool_constant<                            \
+    test_ ## name ## 2 <Vector> (std::bool_constant<                            \
         restriction<typename Vector::value_type>::value &&                 \
         is_copy_constructible<typename Vector::value_type>::value          \
       >());                                                                \
@@ -933,7 +931,7 @@ uint64_t ReadTSC() {
     return true;                                                           \
   }                                                                        \
   template <class Vector> bool test_I_ ## name ## 3 () {                   \
-    return test_I_ ## name ## 2 <Vector> (bool_constant<                   \
+    return test_I_ ## name ## 2 <Vector> (std::bool_constant<                   \
       restriction<typename Vector::value_type>::value>());                 \
     return false;                                                          \
   }                                                                        \
@@ -1176,8 +1174,7 @@ struct PrettyType<Alloc<T>> {
   }                                                                   \
   if (ticks >= 0 && Ticker::CountTicks > ticks && !ticks_thrown)      \
     FAIL() << "CountTicks = " << Ticker::CountTicks << " > " << ticks \
-           << " = ticks"                                              \
-           << ", but no tick error was observed";                     \
+           << " = ticks" << ", but no tick error was observed";       \
   VERIFICATION
 #define CLOSER_ticks }
 
@@ -1338,7 +1335,7 @@ std::pair<typename Vector::iterator, string> iterSpotter(Vector& v, int i) {
         msg = "a[1]";
         break;
       }
-      FOLLY_FALLTHROUGH;
+      [[fallthrough]];
     case 0:
       it = v.begin();
       msg = "a.begin";
@@ -1351,7 +1348,7 @@ std::pair<typename Vector::iterator, string> iterSpotter(Vector& v, int i) {
         msg = "a[-1]";
         break;
       }
-      FOLLY_FALLTHROUGH;
+      [[fallthrough]];
     case 3:
       it = v.end();
       msg = "a.end";
